@@ -10,9 +10,13 @@ import jsonpickle
 
 from .config import config, s3_config
 from .io import S3Artifact
-from .op_template import get_k8s_core_v1_api, get_k8s_dynamic_client
-from .utils import (download_artifact, download_s3, get_key, upload_artifact,
-                    upload_s3)
+from .op_template import get_k8s_client
+from .utils import download_artifact, get_key, upload_s3
+
+try:
+    import kubernetes
+except Exception:
+    pass
 
 logger = logging.getLogger(__name__)
 
@@ -84,16 +88,6 @@ class ArgoParameter(ArgoObjectDict):
                     with open(os.path.join(tmpdir, fs[0]), "r") as f:
                         content = jsonpickle.loads(f.read())
                         self.value = content
-                        # For backward compatibility
-                        # TODO: delete me in the future
-                        if isinstance(content, dict) and "value" in content:
-                            if "type" in content:
-                                self.type = content["type"]
-                            if "type" in content and \
-                                    content["type"] != str(str):
-                                self.value = jsonpickle.loads(content["value"])
-                            else:
-                                self.value = content["value"]
                 except Exception as e:
                     logger.warning("Failed to load parameter value from "
                                    "artifact: %s" % e)
@@ -195,6 +189,7 @@ class ArgoStep(ArgoObjectDict):
             self.outputs.artifacts[name].local_path = s3.local_path
             return
         assert isinstance(s3, S3Artifact), "must provide a S3Artifact object"
+        old_key = get_key(self.outputs.artifacts[name], raise_error=False)
         if s3_config["repo_type"] == "s3":
             self.outputs.artifacts[name].s3 = ArgoObjectDict(s3.to_dict())
         elif s3_config["repo_type"] == "oss":
@@ -206,65 +201,7 @@ class ArgoStep(ArgoObjectDict):
         elif s3.key[-4:] != ".tgz" and not hasattr(self.outputs.artifacts[
                 name], "archive"):
             self.outputs.artifacts[name]["archive"] = {"none": {}}
-
-    def download_sliced_output_artifact(
-            self,
-            name: str,
-            path: os.PathLike = ".",
-    ) -> None:
-        """
-        Download output artifact of a sliced step
-
-        Args:
-            name: artifact name
-            path: local path
-        """
-        assert (hasattr(self, "outputs") and
-                hasattr(self.outputs, "parameters") and
-                "dflow_%s_path_list" % name in self.outputs.parameters), \
-            "%s is not sliced output artifact" % name
-        path_list = jsonpickle.loads(
-            self.outputs.parameters["dflow_%s_path_list" % name].value)
-        for item in path_list:
-            sub_path = item["dflow_list_item"]
-            if config["mode"] == "debug":
-                os.makedirs(os.path.dirname(os.path.join(path, sub_path)),
-                            exist_ok=True)
-                os.symlink(
-                    os.path.join(self.outputs.artifacts[name].local_path,
-                                 sub_path), os.path.join(path, sub_path))
-            else:
-                download_s3(get_key(self.outputs.artifacts[name]) + "/" +
-                            sub_path, path=os.path.join(path, sub_path))
-
-    def upload_and_modify_sliced_output_artifact(
-            self,
-            name: str,
-            path: Union[os.PathLike, List[os.PathLike]],
-    ) -> None:
-        """
-        Upload and modify output artifact of a sliced step
-
-        Args:
-            name: artifact name
-            path: local path to be uploaded
-        """
-        assert (hasattr(self, "outputs") and
-                hasattr(self.outputs, "parameters") and
-                "dflow_%s_path_list" % name in self.outputs.parameters), \
-            "%s is not sliced output artifact" % name
-        path_list = jsonpickle.loads(
-            self.outputs.parameters["dflow_%s_path_list" % name].value)
-        if not isinstance(path, list):
-            path = [path]
-        assert len(path_list) == len(path), "Require %s paths, %s paths"\
-            " provided" % (len(path_list), len(path))
-        path_list.sort(key=lambda x: x['order'])
-        new_path = [None] * (path_list[-1]['order'] + 1)
-        for local_path, item in zip(path, path_list):
-            new_path[item["order"]] = local_path
-        s3 = upload_artifact(new_path, archive=None)
-        self.modify_output_artifact(name, s3)
+        self.outputs.artifacts[name].modified = {"old_key": old_key}
 
     def get_pod(self):
         assert self.type == "Pod"
@@ -273,11 +210,14 @@ class ArgoStep(ArgoObjectDict):
         template_name = self.templateName
         node_id = self.id
         pod_name = get_pod_name(wf_name, node_name, template_name, node_id)
-        core_v1_api = get_k8s_core_v1_api()
-        self.pod = core_v1_api.api_client.call_api(
-            '/api/v1/namespaces/%s/pods/%s' % (config["namespace"], pod_name),
-            'GET', response_type='V1Pod', header_params=config["http_headers"],
-            _return_http_data_only=True)
+        with get_k8s_client() as k8s_client:
+            core_v1_api = kubernetes.client.CoreV1Api(k8s_client)
+            self.pod = core_v1_api.api_client.call_api(
+                '/api/v1/namespaces/%s/pods/%s' % (config["namespace"],
+                                                   pod_name),
+                'GET', response_type='V1Pod',
+                header_params=config["http_headers"],
+                _return_http_data_only=True)
         return self.pod
 
     def get_script(self):
@@ -315,37 +255,39 @@ class ArgoStep(ArgoObjectDict):
         if self.pod is None:
             self.get_pod()
         self.pod.metadata.resource_version = None
-        core_v1_api = get_k8s_core_v1_api()
-        try:
+        with get_k8s_client() as k8s_client:
+            core_v1_api = kubernetes.client.CoreV1Api(k8s_client)
+            try:
+                core_v1_api.api_client.call_api(
+                    '/api/v1/namespaces/%s/pods/%s' % (config["namespace"],
+                                                       self.pod.metadata.name),
+                    'DELETE', response_type='V1Pod',
+                    header_params=config["http_headers"],
+                    _return_http_data_only=True)
+            except Exception as e:
+                logging.warn("Failed to delete pod", e)
             core_v1_api.api_client.call_api(
-                '/api/v1/namespaces/%s/pods/%s' % (config["namespace"],
-                                                   self.pod.metadata.name),
-                'DELETE', response_type='V1Pod',
+                '/api/v1/namespaces/%s/pods' % config["namespace"],
+                'POST', body=self.pod, response_type='V1Pod',
                 header_params=config["http_headers"],
                 _return_http_data_only=True)
-        except Exception as e:
-            logging.warn("Failed to delete pod", e)
-        core_v1_api.api_client.call_api(
-            '/api/v1/namespaces/%s/pods' % config["namespace"],
-            'POST', body=self.pod, response_type='V1Pod',
-            header_params=config["http_headers"], _return_http_data_only=True)
-        try:
-            dynamic_client = get_k8s_dynamic_client()
-            dynamic_client.patch(
-                dynamic_client.resources.get(
-                    group="argoproj.io", api_version="v1alpha1",
-                    kind="Workflow"),
-                body={
-                    "metadata": {
-                        "namespace": config["namespace"],
-                        "name": self.workflow,
+            try:
+                dynamic_client = kubernetes.dynamic.DynamicClient(k8s_client)
+                dynamic_client.patch(
+                    dynamic_client.resources.get(
+                        group="argoproj.io", api_version="v1alpha1",
+                        kind="Workflow"),
+                    body={
+                        "metadata": {
+                            "namespace": config["namespace"],
+                            "name": self.workflow,
+                        },
+                        "status": {"nodes": {self.id: {"phase": "Running"}}},
                     },
-                    "status": {"nodes": {self.id: {"phase": "Running"}}},
-                },
-                content_type="application/merge-patch+json",
-                header_params=config["http_headers"])
-        except Exception as e:
-            logging.warn("Failed to patch workflow", e)
+                    content_type="application/merge-patch+json",
+                    header_params=config["http_headers"])
+            except Exception as e:
+                logging.warn("Failed to patch workflow", e)
 
 
 max_k8s_resource_name_length = 253
